* Program: 4GpStratPWlogrank.sas               
* Author:  John M. Lachin <jml@bsc.gwu.edu>                  
*          Copyright (c) 2015 by John M. Lachin                      
* Purpose: This program determines the Power for K group stratified analysis of log hazard ratios
* Web URL: http://www.bsc.gwu.edu/               
*--------------------------------------------------------------------*;
options ls=130;
**;
**    to run this program, the steps are:
**       1.  specify the values of the macro variables;
**       2.  create a data set called indata;
**       3.  invoke the macro %ssn;
**;
**    macro variables:;
**       title = a text description of the design
**       alpha = two-sided type i error probability;
**       qe = sample fractions for the experimental group,;
**            assumed the same for all strata;
**       qc = sample fractions for the control group,;
**            assumed the same for all strata, q**= 1.0 - &qe;
**       power = total power;
**;
**    data set variables, one record per study or scenario;
**       r = period of recruitment;
**       t = total period of study;
**       lc = hazard rate (lc) for the control group;
**       rr = relative hazard in treaed group, le = rr*lc;
**       ldc = hazard rate for losses to followup in the control group;
**       lde = hazard rate for losses to followup in the experimental group;
**;
**;
 
%macro  ev(rr,tt,l,ht,g);
   gm = &l*&g*exp(-&ht*&tt)*(1 - exp((&ht - &g)*&rr));
   ev = (&l/&ht) + gm/   ( (1 - (exp(-&g*&rr))) *&ht*(&ht - &g) );
%mend;
 
%macro STRATPARM (stratum);
 
title2 'stratum parameters for exponential hazards with exponential losses';
 
** N = the stratum sample size;
 
data dataone; set indata;
array ls {&K} l1 - l&K;
array Qs {&K} Q1 - Q&K;
array pds {&K} pd1 - pd&K;
array pls {&K} pl1 - pl&K;
array ds {&K} d1 - d&K;
array los {&K} lo1 - lo&K;
array rrs {&K} rr1 - rr&K;
 
dtot = 0;
ls(1) = l1;
do j = 2 to &K;
ls{j} = rrs{j}*l1; ******* group 1 is the reference;
end;
 
do j = 1 to &K;
**** event probabilities under the alternative;
  le = ls{j};
  te = le + ld;
      %ev(r,t,le,te, gamma);
      pds{j} = ev; **** event probability;
	  ds{j} = &N*qs{j}*ev;  
      dtot = dtot + ds{j};
**** loss probabilities;
      %ev(r,t,ld,te, gamma);
      pls{j} = ev;
	  los{j} = &N*qs{j}*ev;;
end;
 
df = &K -1;
chia = cinv(1-&alpha,df);
*chinc = cnonct(chia,df,1-&power);
 
data datatwo; set dataone;
array ls {&K} l1 - l&K;
array Qs {&K} Q1 - Q&K;
array pds {&K} pd1 - pd&K;
array pls {&K} pl1 - pl&K;
array ds {&K} d1 - d&K;
array los {&K} lo1 - lo&K;
array thetas{&K} theta1 - theta&K;
 
ncp = 0; thetabar = 0;
do j = 1 to &K;
  thetas{j} = log(ls{j});
  thetabar = thetabar + ds{j}*thetas{j};
end;
thetabar = thetabar/dtot;
 
do j = 1 to &K;
  ncp = ncp + ds{j}*(thetas{j} - thetabar)**2;
end;
 
beta = probchi(chia,df,ncp);
power = 1 - beta;
 
N = &N;
proc print; var gamma N d1-d&K l1-l&K rr1-rr&K /*chia df ncp beta*/ power;
 
data &stratum; set datatwo (keep = l1-l&K rr2-rr&K d1-d&K);
array ds {&K} d1 - d&K;
array rrs {&K} rr1 - rr&K;
array ls{&K} l1 - l&K;
 
**** generate a data set of the form used with the old pcstrat program,
in = 1 designates the log relative risk vector elements for j = 1 to (K-1) groups 
       versus the reference (i = 0 for all records),
in = 2 designates the covariance matrix of the log relative risk estimates with elements
     for the ith versus the reference and the jth versus the reference,
x = the value of either the log rr or the covariance matrix element;
 
keep in i j x; 
 
do k = 2 to &K;
 in = 1; i = 0; j = k - 1;
 x = log(ls(k)) - log(ls(1)); ********* group 1 is the reference;
output;
end;
do i = 1 to (&K-1);
do j = 1 to (&K-1);
 in = 2;
 if i = j then x = (1/ds(1)) + (1/ds(i+1));
 if i ne j then x = 1/ds(1);
output;
end; end;
 
proc print data = &stratum;
run;
 
%mend;
 
/**************************************************************/                
/*                     MultMVLE                               */                
/*                   ============                             */                
/*               THE BIOSTATISTICS CENTER                     */                
/*           THE GEORGE WASHINGTON UNIVERSITY                 */                
/*============================================================*/                
/*   PLEASE FOLLOW THE INSTRUCTIONS AS BELOW :                */                
/*  (1) Specify the data sets names for each stratum  	  */
/*      AN EXAMPLE OF LEGAL CALLS :                           */                
/*                  STRATIFY(WY WN);                          */                
/*      WILL PERFORM THE STRATIFIED ANALYSIS FOR WY WN STRATUM*/
/*  (2) include a specification of the number of groups (K)   */
/*      being compared within each stratum such as         	  */
/*         %let K = 5; 						    	  */
/*      in the sas program before calling this macro       	  */
/*  (3) Each data set must contain the variables in, i, j, x  */
/*       in = 1 if a statistic, =2 if a variance or covariance*/
/*       if in = 1 then i = 0, j = stratum, x = statistic     */
/*       if in = 2 and i = j then x = the variance,			  */
/*       if in = 2 and i ne j then x = the covariance.        */ 
/**************************************************************/                
                                                                                
OPTIONS LS=79;/* MACROGEN SYMBOLGEN MLOGIC;*/                                   
 
/*****************************************************************/             
%MACRO NAMES(STR);                                                        
 %DO T=1 %TO (&K - 1);                                                           
 S&T._1                                                                         
 %END;                                                                          
%MEND NAMES;                                                                    
                                                                                
/************************************************************/                  
%MACRO STRATIFY(STRATA);                                           
                                                                                
%PUT **TEST FOR HOMOGENEITY OF POOLED WITHIN THETAS ACROSS STRATUM**;           
/* SCREEN OUT THE ELEMENTS OF INPUT STRING. */                                  
                                                                                
%LET I=1;                                                                       
%LET STR&I=%SCAN(&STRATA,&I);                                                   
%DO %WHILE(&&STR&I NE);                                                         
   %LET I = %EVAL(&I + 1);                                                      
   %LET STR&I=%SCAN(&STRATA,&I);                                                
   %PUT **STR&I=&&STR&I**;                                                      
%END;                                                                           
                                                                                
%LET NUMSTR = %EVAL(&I-1);                                                      
%PUT **NUMSTR=&NUMSTR**;                                                        
                                                                                
%DO I = 1 %TO &NUMSTR;                                                          
%LET STR = &&STR&I;                                                             
  %PUT **STR=&STR K=&K NUMSTR=&NUMSTR **;                             
                                                                                
/****** READ IN DATA SET *****************************************/             
 DATA T_1 %NAMES(&STR);                                       
set &STR END=LAST;                                                           
RETAIN THETAOK 0;                                                               
  KEEP X;                                                                       
                                                                                
  IF IN = 1 THEN DO;                                                       
     THETAOK=1;                                                                 
     OUTPUT T_1;                                                                
  END;                                                                          
  IF IN = 2 THEN DO;                                                         
   %DO R=1 %TO (&K - 1);                                                         
    IF I = &R THEN OUTPUT S&R._1;/*ADD ROWS OF VAR-COV MATRIX*/;                
   %END;                                                                        
  END;                                                                          
                                                                                
/*********FETCH*************************************************/               
DATA FF;                                                                        
PROC IML;                                                                       
RESET NONAME NOLOG;                                                             
USE T_1;                                                                        
  READ ALL INTO T1;                                                             
    T1 = T1`;                                                                   
USE S1_1;                                                                       
  READ ALL INTO S1;                                                             
%DO P=2 %TO (&K - 1);                                                            
USE S&P._1;                                                                     
  READ ALL INTO S1&P;                                                           
  S1 = S1 || S1&P;                                                              
  %END;                                                                         
                                                                                
PRINT "PCSTRAT";                                                                
PRINT "=================================";                                      
PRINT "THE BIOSTATISTICS CENTER";                                               
PRINT "THE GEORGE WASHINGTON UNIVERSITY ",;                                     
                                                                                
     TP1 = T1`;                                                                 
     SP1 = S1 ;                                                                 
     ISP1 = INV(SP1);                                                           
                                                                                
PRINT "==========================================================",;            
PRINT " MULTIVARIATE ANALYSIS";                                           
PRINT " (THIS IS FOR &STR STRATA) ",;                                           
PRINT "==========================================================",;            
PRINT "STATISTICS :",;                                                    
PRINT TP1,;                                                                     
PRINT "THE COVARIANCE MATRIX :",;                                               
PRINT SP1,;                                                                     
PRINT "THE INVERSE OF COVARIANCE MATRIX :",;                                    
PRINT ISP1,;                                                                    
CREATE TP_&STR FROM TP1;                                                        
APPEND FROM TP1;                                                                
CREATE SP_&STR FROM SP1;                                                        
APPEND FROM SP1;                                                                
CREATE ISP_&STR FROM ISP1;                                                      
APPEND FROM ISP1;                                                               
%END;                                                                              
                                                                                
FREE T1 S1;
 
/******** POOLED ALL THETA ACROSS STRATA ******/                                
%DO Q=1 %TO &NUMSTR;                                                            
USE TP_&&STR&Q;                                                                 
READ ALL INTO TP&&STR&Q;                                                        
USE SP_&&STR&Q;                                                                 
READ ALL INTO SP&&STR&Q;                                                        
%END;                                                                           
T&STR1 = TP&STR1;                                                               
S&STR1 = SP&STR1;                                                               
                                                                                
%DO W=2 %TO &NUMSTR;                                                            
     T&STR1 = T&STR1 // TP&&STR&W;                                              
     S&STR1 = BLOCK(S&STR1,SP&&STR&W);                                          
%END;                                                                           
     TP = T&STR1;                                                               
     TPA= TP`;                                                                  
     SP = S&STR1;                                                               
START LOO;                                                                      
IF &NUMSTR = 2 THEN                                                             
     BA = I(NROW(TP&STR1)) || -I(NROW(TP&STR2));                                
IF &NUMSTR > 2 THEN DO;                                                         
     C1&STR1=-I(NROW(TP&STR1));                                                 
     %DO Y=2 %TO &NUMSTR-1;                                                     
        C1&STR1 = C1&STR1 // -I(NROW(TP&&STR&Y));                               
     %END;                                                                      
     BA = I(NROW(C1&STR1)) || C1&STR1;                                          
END;                                                                            
FINISH;                                                                         
*RUN LOO;                                                                        
     DF = &K - 1; *** number of rrs = # groups - 1;                                            
                                                                                
PRINT "MEASURE-SPECIFIC WITHIN STRATUM THETA ESTIMATES:",;                      
PRINT TPA,;                                                                     
PRINT "THE COVARIANCE MATRIX :",;                                               
PRINT SP,;                                                                      
FREE T&STR1 S&STR1 TP SP BA C1&STR1;                         
 
/*********** POOLED ESTIMATORS OVER STRATA *************/                       
%DO Q=1 %TO &NUMSTR;                                                            
USE TP_&&STR&Q;                                                                 
READ ALL INTO TP&&STR&Q;                                                        
USE SP_&&STR&Q;                                                                 
READ ALL INTO SP&&STR&Q;                                                        
USE ISP_&&STR&Q;                                                                
READ ALL INTO ISP&&STR&Q;                                                       
                                                                                
%END;                                                                           
                                                                                
I&STR1 = ISP&STR1;                                                              
%DO V=2 %TO &NUMSTR;                                                            
     I&STR1 = I&STR1 + ISP&&STR&V;                                              
%END;                                                                           
     TISP = I&STR1;                                                             
     ITISP = INV( TISP);                                                        
%DO F=1 %TO &NUMSTR;                                                            
     A&F = ITISP * ISP&&STR&F;                                                  
%END;                                                                           
C&STR1=A1;                                                                      
%DO E=2 %TO &NUMSTR;                                                            
     C&STR1 = C&STR1 + A&E;                                                     
%END;                                                                           
     CHECK = C&STR1;                                                            
T1&STR1 = A1 * TP&STR1;                                                         
%DO L=2 %TO &NUMSTR;                                                            
     T1&STR1 = T1&STR1 + ( A&L * TP&&STR&L );                                   
%END;                                                                           
     TPT = T1&STR1;                                                             
     TPTT=TPT`;                                                                 
S1&STR1 = A1 * SP&STR1 * A1`;                                                   
%DO L1=2 %TO &NUMSTR;                                                           
     S1&STR1 = S1&STR1 + ( A&L1 * SP&&STR&L1 * A&L1`);                          
%END;                                                                           
     SPT = S1&STR1;                                                             
*     SESP = SQRT( SPT);
     Ispt = inv(spt);  
                                                                                
PRINT "==========================================================",;            
PRINT "STRATIFIED-ADJUSTED ANALYSIS COMBINED OVER STRATA",;                     
PRINT "==========================================================",;            
PRINT "ESTIMATES JOINTLY COMBINED OVER STRATA:",;                               
PRINT TPTT,;                                                                    
PRINT "THE COVARIANCE MATRIX :",;                                               
PRINT SPT,;                                                                     
PRINT "THE INVERSE OF COVARIANCE MATRIX :",;                                    
PRINT ispt,;                                                                    
PRINT "THE JOINTLY COMBINED ODDS RATIOS OVER STRATA :",;                                    
or = exp(tptt);
print or;
NCP = TPTT * ispt * TPTT`;
chia = cinv({1}-{&alpha}, df); 
power = {1} - probchi(chia, df, ncp);
PRINT "Df for the stratified adjusted K-group test:" df,;
print "Chi-square critical value at level alpha:  " chia,;
print "Non-centrality parameter for the test:     " ncp,; 
print "Power                                      " power,;
 
ncpH = 0; **** HOMOGENEITY CHI-SQUARE NCP;
 
%DO F=1 %TO &NUMSTR;  
tht&F=TP&&STR&F`;
invv&F=ISP&&STR&F; 
%END;    
 
%DO I = 1 %TO &NUMSTR;                                                          
print "parameters within stratum (tht&i) =" tht&i;
print "within stratum inverse covariance matris (invv&i) =" invv&i;
ncpH = ncpH + (tht&i - TPTT)*invv&i*(tht&i - TPTT)`;                      
%END;                                                                              
 
DF = (&K - 1) * (&NUMSTR - 1);
chia = cinv({1}-{&alpha}, df); 
power = {1} - probchi(chia, df, ncph);
PRINT "Df for the test of group x stratum interaction:" df,;
print "Chi-square critical value at level alpha:      " chia,;
print "Non-centrality parameter for the test:         " ncph,; 
print "Power                                          " power,;
 
%MEND STRATIFY;   
run;
 
*****************************************************************************************************
 
INPUT SPECIFICATIONS AND CALLS TO MACROS;
 
*****************************************************************************************************;
 
 
%let title = diabetes treatment study;
%let alpha = 0.05;  *** two sided value;
%let k = 4;
 
TITLE1 "Stratified analysis with little heterogeneity between strata";
data one;
r = 3; t = 7;
ld = 0.04; *** assumed the same among groups, could modify;
 
q1 = 1/4;
q2 = 1/4;
q3 = 1/4;
q4 = 1/4;
 
gamma = -0.27;
l1 = 0.0875;  **** group 1 is the reference; 
rr1 = 1;
RR2 = 1;
RR3 = 1;
RR4 = 0.75;
 
data indata; set one;
%let N = 3000;
%stratparm (strat1); 
run;
**** if the stratum data set names are changed from strat1 and strat2, 
                           then the call to 
                           macro stratify must be changed to use the new names;  
 
data indata; set one;
l1 = 0.07;  **** group 1 is the reference; 
RR4 = 0.85;  ********* NOTE;
%let N = 2000;
%stratparm (strat2);  
run;
 
**** if the stratum data set names are changed from strat1 and strat2, 
                           then the call to 
                           macro stratify must be changed to use the new names;  
                                                                             
%STRATIFY(strat1 strat2);                                                          
run;                                                                                
 
******************************************************************************************;
 
TITLE1 "Test of treatment group by strata interaction, e.g. subgroup heterogeneity, with TWO strata";
 
data indata; set one;
RR4 = 0.75*0.75;
 
%let N = 2500;
%stratparm (strat1); 
run;
**** if the stratum data set names are changed from strat1 and strat2, 
                           then the call to 
                           macro stratify must be changed to use the new names;  
 
data indata; set one;
RR4 = 0.75*1.25;
%let N = 2500;
%stratparm (strat2);  
run;
 
**** if the stratum data set names are changed from strat1 and strat2, 
                           then the call to 
                           macro stratify must be changed to use the new names;  
                                                                             
%STRATIFY(strat1 strat2);                                                          
run;                                                                                
 
******************************************************************************************;
 
TITLE1 "Test of treatment group by strata interaction, e.g. subgroup heterogeneity, with THREE strata";
data indata; set one;
RR4 = 0.75*0.75;
%let N = 5000/3;
%stratparm (strat1); 
run;
**** if the stratum data set names are changed from strat1 and strat2, 
                           then the call to 
                           macro stratify must be changed to use the new names;  
 
data indata; set one;
RR4 = 0.75;
%stratparm (strat2);  
run;
 
data indata; set one;
RR4 = 0.75*1.25;
%stratparm (strat3);  
run;
 
%STRATIFY(strat1 strat2 strat3);                                                          
run;                                                                                
 
 