*--------------------------------------------------------------------
* Program: 4gpexpwlogl z.sas                 
* Author:  John M. Lachin <jml@bsc.gwu.edu>                  
*          Copyright (c) 2015 by John M. Lachin                      
* Purpose: This program computes the power of a K-group test of 
           of homogeneity using Cochran's test
* Web URL: http://www.bsc.gwu.edu/                 
*--------------------------------------------------------------------*;
 
options ls=120 formdlim = '-';
**;
**    to run this program, the steps are:
**       1.  specify the values of the macro variables;
**       2.  create a data set called indata;
**       3.  invoke the macro %ssn;
**;
**    macro variables:;
**       title = a text description of the design
**       alpha = two-sided type i error probability;
**       qe = sample fractions for the experimental group,;
**            assumed the same for all strata;
**       qc = sample fractions for the control group,;
**            assumed the same for all strata, q**= 1.0 - &qe;
**       power = total power;
**;
**    data set variables, one record per study or scenario;
**       r = period of recruitment;
**       t = total period of study;
**       lc = hazard rate (lc) for the control group;
**       rr = relative hazard in treaed group, le = rr*lc;
**       ldc = hazard rate for losses to followup in the control group;
**       lde = hazard rate for losses to followup in the experimental group;
**;
**;
 
%macro  ev(rr,tt,l,ht,g);
   gm = &l*&g*exp(-&ht*&tt)*(1 - exp((&ht - &g)*&rr));
   ev = (&l/&ht) + gm/   ( (1 - (exp(-&g*&rr))) *&ht*(&ht - &g) );
%mend;
 
title1 'sample size for ratio of exponential hazards with exponential losses';
 
%macro power;
data newone; set indata;
array ls {&K} l1 - l&K;
array Qs {&K} Q1 - Q&K;
array pds {&K} pd1 - pd&K;
array pls {&K} pl1 - pl&K;
array eds {&K} ed1 - ed&K;
array elos {&K} elo1 - elo&K;
array rrs {&K} rr1 - rr&K;
 
do j = 1 to &K;
ls{j} = rrs{j}*lref;
end;
 
do j = 1 to &K;
  qs{j} = 1/&K;
**** event probabilities under the alternative;
  le = ls{j};
  te = le + ld;
      %ev(r,t,le,te, gamma);
      pds{j} = ev; **** probability of the event;
	  eds{j} = &N*qs{j}*ev; **** expected number of events;  
**** loss probabilities;
      %ev(r,t,ld,te, gamma);
      pls{j} = ev;
	  elos{j} = &N*qs{j}*ev; *** expected number of losses to follow-up;
end;
 
df = &K -1;
chia = cinv(1-&alpha,df);
*chinc = cnonct(chia,df,1-&power);
 
data two; set newone;
array ls {&K} l1 - l&K;
array Qs {&K} Q1 - Q&K;
array pds {&K} pd1 - pd&K;
array pls {&K} pl1 - pl&K;
array Eds {&K} ed1 - ed&K;
array elos {&K} elo1 - elo&K;
array thetas{&K} theta1 - theta&K;
array betas{&K} beta1 - beta&K;
array rrs {&K} rr1 - rr&K;
 
N = &N;
 
totevt = 0; totlos = 0;
do j = 1 to &K;
  totevt = totevt + eds(j);
  totlos = totlos + elos(j);
end;
 
ncp = 0; thetabar = 0;
do j = 1 to &K;
  thetas{j} = log(ls{j});
  thetabar = thetabar + eds{j}*thetas{j};
end;
thetabar = thetabar/totevt;
 
do j = 1 to &K;
  ncp = ncp + eds{j}*(thetas{j} - thetabar)**2;
end;
 
avehaz = exp(thetabar);
 
beta = probchi(chia,df,ncp);
power = 1 - beta;
 
*** JL total events from Cox Score test, group 1 as reference;
do j = 1 to &K;
  betas(j)=log(rrs(j));
end;
*** group 1 as reference;
PHSncp = 0; *** non centrality parameter for PH score test;
do j = 2 to &K;
phsncp = phsncp + totevt*Qs(j)*(1-Qs(j))*(betas(j)**2);
end;
 
do j = 2 to &K-1;
do k = (j+1) to &K;
phsncp = phsncp - 2*totevt*Qs(j)*Qs(k)*betas(j)*betas(k);
end; end;
phsPwr = 1 - probchi(chia,df,phsncp);
 
**** Note that the result from the score test equals the Ahnn-Anderson (1995) expression when there 
are equal sample sizes and equal censoring;
 
proc print; var R T gamma pd1-pd&K pl1-pl&K lref rr1-rr&K beta1-beta&K N  
  ed1-ed&K df ncp power thetabar avehaz totevt totlos phsncp phspwr;
run;
%mend;
 
 run;
 
%let title = diabetes treatment study;
%let alpha = 0.05;  *** two sided value;
%let N = 3300;  **** omnibus test n;
%let K = 4; **** specify the number of groups;
 
data one;
r = 3; T = 7;
ld = 0.04; *** assumed the same among groups, could modify;
 
gamma = -0.27; 
 
***** lref = hazard in whichever group is the reference;
 
do lref = 0.0875; 
 
***** RRj = relative risk for the j-th group versus the reference = 1 when j = reference;
***** these must be specified specifically for the specified number of groups in the %let statement above;
rr1 = 1;
RR2 = 1;
RR4 = 0.75;
RR3 = 1;
 output;
end; 
title2 "CHECK run, group 1 as the reference, group 4 lower risk than groups 1 - 3";
data indata; set one;
%power;
run;
 
data indata; set indata;
%let N = 5000;  *** N for 6 pairwise tests;
%power; run;
title2 "Onmibus power with pairwise N, group 1 as the reference, group 4 lower risk than groups 1 - 3";
 
data indata; set one;
Do RR4 = 0.79 to 0.81 by 0.001;
output;
end;
title2 "group 1 as the reference, group 4 with smaller relative risk";
%power; run;
proc print; var N ncp rr4 power;
run;
 
data indata; set one;
lref = (3/5)*0.0875 + (2/5)*0.07;
lrr = (3/5)*log(0.75) + (2/5)*log(0.85);
RR4 = exp(lrr);
 
title2 "non-stratified computation for 2 strata example in discussion";
%power; run;
 