*--------------------------------------------------------------------*;
* Program: 4gpexSSlogl z:                
* Author:  John M. Lachin <jml@bsc.gwu.edu>                  
*          Copyright (c) 2015 by John M. Lachin                      
* Purpose: Computes the sample size for the K-group test. 
*          Allows for non-linear recruitment            
* Web URL: http://www.bsc.gwu.edu/                 
*--------------------------------------------------------------------*;
options ls=120 formdlim = '-';
**;
**    to run this program, the steps are:
**       1.  specify the values of the macro variables;
**       2.  create a data set called indata;
**       3.  invoke the macro %ssn;
**;
**    macro variables:;
**       title = a text description of the design
**       alpha = two-sided type i error probability;
**       qe = sample fractions for the experimental group,;
**            assumed the same for all strata;
**       qc = sample fractions for the control group,;
**            assumed the same for all strata, q**= 1.0 - &qe;
**       power = total power;
**;
**    data set variables, one record per study or scenario;
**       r = period of recruitment;
**       t = total period of study;
**       lc = hazard rate (lc) for the control group;
**       rr = relative hazard in treaed group, le = rr*lc;
**       ldc = hazard rate for losses to followup in the control group;
**       lde = hazard rate for losses to followup in the experimental group;
**;
**;
 
%macro  ev(rr,tt,l,ht,g);
   gm = &l*&g*exp(-&ht*&tt)*(1 - exp((&ht - &g)*&rr));
   ev = (&l/&ht) + gm/   ( (1 - (exp(-&g*&rr))) *&ht*(&ht - &g) );
%mend;
 
title1 'sample size for ratio of exponential hazards with exponential losses';
 
%macro ssn;
data newone; set indata;
array ls {&K} l1 - l&K;
array Qs {&K} Q1 - Q&K;
array pds {&K} pd1 - pd&K;
array pls {&K} pl1 - pl&K;
array ds {&K} d1 - d&K;
array los {&K} lo1 - lo&K;
array rrs {&K} rr1 - rr&K;
 
dtot = 0;
do j = 1 to &K;
ls{j} = rrs{j}*lref;
end;
 
do j = 1 to &K;
  qs{j} = 1/&K;
**** event probabilities under the alternative;
  le = ls{j};
  te = le + ld;
      %ev(r,t,le,te, gamma);
      pds{j} = ev; **** probability of the event;
	  ds{j} = qs{j}*ev;  *** term not multiplied by N;
      dtot = dtot + ds{j};
**** loss probabilities;
      %ev(r,t,ld,te, gamma);
      pls{j} = ev;
	  los{j} = qs{j}*ev;  *** term not multiplied by N;
end;
 
df = &K -1;
chia = cinv(1-&alpha,df);
chinc = cnonct(chia,df,1-&power);
 
data two; set newone;
array ls {&K} l1 - l&K;
array Qs {&K} Q1 - Q&K;
array pds {&K} pd1 - pd&K;
array pls {&K} pl1 - pl&K;
array ds {&K} d1 - d&K;
array Eds {&K} ed1 - ed&K;
array los {&K} lo1 - lo&K;
array thetas{&K} theta1 - theta&K;
array betas{&K} beta1 - beta&K;
array rrs {&K} rr1 - rr&K;
 
ncf = 0; thetabar = 0;
do j = 1 to &K;
  thetas{j} = log(ls{j});
  thetabar = thetabar + ds{j}*thetas{j};
end;
thetabar = thetabar/dtot;
 
do j = 1 to &K;
  ncf = ncf + ds{j}*(thetas{j} - thetabar)**2;
end;
 
avehaz = exp(thetabar);
N = (chinc/ncf);
power = &power;
 
totevt = 0;
do j = 1 to &K;
  eds(j) = n*ds{j};
totevt = totevt + eds(j);
end;
 
*** JL total events from Cox Score test, group 1 as reference;
do j = 1 to &K;
  betas(j)=log(rrs(j));
end;
*** group 1 as reference;
PHSncf = 0; *** non central factor for PH score test;
do j = 2 to &K;
phsncf = phsncf + Qs(j)*(1-Qs(j))*(betas(j)**2);
end;
 
do j = 2 to &K-1;
do k = (j+1) to &K;
phsncf = phsncf - 2*Qs(j)*Qs(k)*betas(j)*betas(k);
end; end;
phsevt = chinc/phsncf;
 
**** Note that the result from the score test equals the Ahnn-Anderson (1995) expression when there 
are equal sample sizes and equal censoring;
 
proc print; var R T gamma pd1-pd&K pl1-pl&K d1-d&K lref rr1-rr&K beta1-beta&K chinc df ncf power thetabar avehaz N 
  ed1-ed&K totevt phsncf phsevt;
run;
%mend;
 
 run;
 
%let title = diabetes treatment study;
%let alpha = 0.05;  *** two sided value;
%let power=0.90;
%let K = 4; **** specify the number of groups;
 
data one;
r = 3; T = 7;
ld = 0.04; *** assumed the same among groups, could modify;
 
gamma = -0.27; 
 
***** lref = hazard in whichever group is the reference;
 
do lref = 0.0875; 
 
***** RRj = relative risk for the j-th group versus the reference = 1 when j = reference;
***** these must be specified specifically for the specified number of groups in the %let statement above;
rr1 = 1;
RR2 = 1;
RR4 = 0.75;
RR3 = RR4;
 output;
end; 
 
 
title2 "group 1 as the reference, groups 3 and 4 lower risk than groups 1 - 2";
data indata; set one;
%ssn;
run;
 
data indata; set indata;
 
RR3 = 1;
title2 "group 1 as the reference, group 4 lower risk than groups 1 - 3";
%ssn; run;
 
data indata; set one;
rr1 = 1;
rr2 = 0.9;
rr3 = 0.8;
rr4 = 0.7;
 
title2 "group 1 as the reference, range of relative risks";
%ssn; run;
 